@@ -173,6 +173,14 @@ SUPPORTED_DHCP_OPTS_MAPPING = {
         '23': 'dns_server'},
 }
 
+# OVN string type DHCP options
+OVN_STR_TYPE_DHCP_OPTS = [
+    'domain_name',
+    'bootfile_name',
+    'path_prefix',
+    'wpad',
+    'tftp_server']
+
 # Special option for disabling DHCP via extra DHCP options
 DHCP_DISABLED_OPT = 'dhcp_disabled'
 
@@ -359,7 +367,6 @@ LSP_OPTIONS_MCAST_FLOOD = 'mcast_flood'
 LRP_OPTIONS_RESIDE_REDIR_CH = 'reside-on-redirect-chassis'
 
 # Port Binding types
-PB_TYPE_PATCH = 'patch'
 PB_TYPE_VIRTUAL = 'virtual'
 
 HA_CHASSIS_GROUP_DEFAULT_NAME = 'default_ha_chassis_group'
