@@ -160,6 +160,10 @@ def validate_port_extra_dhcp_opts(port):
         invalid_ipv6=invalid[const.IP_VERSION_6] if failed else [])
 
 
+def is_dhcp_option_quoted(opt_value):
+    return opt_value.startswith('"') and opt_value.endswith('"')
+
+
 def get_lsp_dhcp_opts(port, ip_version):
     # Get dhcp options from Neutron port, for setting DHCP_Options row
     # in OVN.
@@ -193,6 +197,9 @@ def get_lsp_dhcp_opts(port, ip_version):
                 continue
 
             opt = mapping[edo['opt_name']]
+            if (opt in constants.OVN_STR_TYPE_DHCP_OPTS and
+                    not is_dhcp_option_quoted(edo['opt_value'])):
+                edo['opt_value'] = '"%s"' % edo['opt_value']
             lsp_dhcp_opts[opt] = edo['opt_value']
 
     return (lsp_dhcp_disabled, lsp_dhcp_opts)
@@ -247,6 +254,11 @@ def validate_and_get_data_from_binding_profile(port):
                 ) % constants.PORT_CAP_PARAM
         raise n_exc.InvalidInput(error_message=msg)
 
+    # Note that only the keys mentioned in each parameter set, as defined in
+    # constants.OVN_PORT_BINDING_PROFILE_PARAMS, will be evaluated.
+    #
+    # Any surplus keys provided by Nova will be ignored and pruned from the
+    # Dict returned by this function.
     for pbp_param_set in constants.OVN_PORT_BINDING_PROFILE_PARAMS:
         if pbp_param_set.vnic_type:
             if pbp_param_set.vnic_type != vnic_type:
@@ -262,13 +274,10 @@ def validate_and_get_data_from_binding_profile(port):
                 pass
         if len(param_dict) == 0:
             continue
-        if len(param_dict) != len(param_keys):
+        if param_keys - binding_profile.keys():
             msg = _('Invalid binding:profile. %s are all '
                     'required.') % param_keys
             raise n_exc.InvalidInput(error_message=msg)
-        if (len(binding_profile) != len(param_keys)):
-            msg = _('Invalid binding:profile. too many parameters')
-            raise n_exc.InvalidInput(error_message=msg)
         break
 
     if not param_dict:
@@ -353,12 +362,12 @@ def get_revision_number(resource, resource_type):
 
 
 def remove_macs_from_lsp_addresses(addresses):
-    """Remove the mac addreses from the Logical_Switch_Port addresses column.
+    """Remove the mac addresses from the Logical_Switch_Port addresses column.
 
     :param addresses: The list of addresses from the Logical_Switch_Port.
         Example: ["80:fa:5b:06:72:b7 158.36.44.22",
                   "ff:ff:ff:ff:ff:ff 10.0.0.2"]
-    :returns: A list of IP addesses (v4 and v6)
+    :returns: A list of IP addresses (v4 and v6)
     """
     ip_list = []
     for addr in addresses:
@@ -372,7 +381,7 @@ def get_allowed_address_pairs_ip_addresses(port):
     """Return a list of IP addresses from port's allowed_address_pairs.
 
     :param port: A neutron port
-    :returns: A list of IP addesses (v4 and v6)
+    :returns: A list of IP addresses (v4 and v6)
     """
     return [x['ip_address'] for x in port.get('allowed_address_pairs', [])
             if 'ip_address' in x]
@@ -385,7 +394,7 @@ def get_allowed_address_pairs_ip_addresses_from_ovn_port(ovn_port):
     allowed_address_pairs column using the data in the OVN port.
 
     :param ovn_port: A OVN port
-    :returns: A list of IP addesses (v4 and v6)
+    :returns: A list of IP addresses (v4 and v6)
     """
     addresses = remove_macs_from_lsp_addresses(ovn_port.addresses)
     port_security = remove_macs_from_lsp_addresses(ovn_port.port_security)
@@ -406,6 +415,13 @@ def get_ovn_port_addresses(ovn_port):
     return list(set(addresses + port_security))
 
 
+def get_virtual_port_parents(nb_idl, virtual_ip, network_id, port_id):
+    ls = nb_idl.ls_get(ovn_name(network_id)).execute(check_error=True)
+    return [lsp.name for lsp in ls.ports
+            if lsp.name != port_id and
+            virtual_ip in get_ovn_port_addresses(lsp)]
+
+
 def sort_ips_by_version(addresses):
     ip_map = {'ip4': [], 'ip6': []}
     for addr in addresses:
