@@ -13,6 +13,8 @@
 #
 
 import abc
+import copy
+import datetime
 
 from oslo_config import cfg
 from oslo_utils import timeutils
@@ -23,6 +25,12 @@ from neutron.common.ovn import utils as ovn_utils
 from neutron.common import utils
 
 
+class DeletedChassis(object):
+    external_ids = {}
+    hostname = '("Chassis" register deleted)'
+    name = '("Chassis" register deleted)'
+
+
 class NeutronAgent(abc.ABC):
     types = {}
 
@@ -37,7 +45,21 @@ class NeutronAgent(abc.ABC):
 
     def update(self, chassis_private, updated_at=None, clear_down=False):
         self.chassis_private = chassis_private
-        self.updated_at = updated_at or timeutils.utcnow(with_timezone=True)
+        if not updated_at:
+            # When use the Chassis_Private table for agents health check,
+            # chassis_private has attribute nb_cfg_timestamp.
+            # nb_cfg_timestamp: the timestamp when ovn-controller finishes
+            # processing the change corresponding to nb_cfg(
+            # https://www.ovn.org/support/dist-docs/ovn-sb.5.html).
+            # it can better reflect the status of chassis.
+            # nb_cfg_timestamp is milliseconds, need to convert to datetime.
+            if hasattr(chassis_private, 'nb_cfg_timestamp'):
+                updated_at = datetime.datetime.fromtimestamp(
+                    chassis_private.nb_cfg_timestamp / 1000,
+                    datetime.timezone.utc)
+            else:
+                updated_at = timeutils.utcnow(with_timezone=True)
+        self.updated_at = updated_at
         if clear_down:
             self.set_down = False
 
@@ -45,9 +67,12 @@ class NeutronAgent(abc.ABC):
     def chassis_from_private(chassis_private):
         try:
             return chassis_private.chassis[0]
-        except (AttributeError, IndexError):
+        except AttributeError:
             # No Chassis_Private support, just use Chassis
             return chassis_private
+        except IndexError:
+            # Chassis register has been deleted but not Chassis_Private.
+            return DeletedChassis
 
     @property
     def chassis(self):
@@ -205,7 +230,10 @@ class AgentCache:
         self.driver = driver
 
     def __iter__(self):
-        return iter(self.agents.values())
+        # Copying self.agents will avoid any issue during the iteration if an
+        # agent is added or deleted.
+        _agents = copy.copy(self.agents)
+        return iter(_agents.values())
 
     def __getitem__(self, key):
         return self.agents[key]
@@ -229,4 +257,13 @@ class AgentCache:
         agent_ids = {cls.id_from_chassis_private(chassis_private)
                      for cls in NeutronAgent.types.values()}
         # Return the cached agents of agent_ids whose keys are in the cache
-        return (self.agents[id_] for id_ in agent_ids & self.agents.keys())
+        return (agent for agent in self if agent.agent_id in agent_ids)
+
+    def get_agents(self, filters=None):
+        filters = filters or {}
+        agent_list = []
+        for agent in self:
+            agent_dict = agent.as_dict()
+            if all(agent_dict[k] in v for k, v in filters.items()):
+                agent_list.append(agent)
+        return agent_list
