@@ -296,10 +296,10 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
             raise RuntimeError(_("Currently only supports "
                                  "delete by lport-name"))
 
-    def get_all_stateful_fip_nats(self):
+    def get_all_stateless_fip_nats(self):
         cmd = self.db_find('NAT',
             ('external_ids', '!=', {ovn_const.OVN_FIP_EXT_ID_KEY: ''}),
-            ('options', '!=', {'stateless': ''}),
+            ('options', '=', {'stateless': 'true'}),
             ('type', '=', 'dnat_and_snat')
         )
         return cmd.execute(check_error=True)
@@ -876,17 +876,6 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
                      card_serial_number)
         raise RuntimeError(msg)
 
-    def get_chassis_data_for_ml2_bind_port(self, hostname):
-        try:
-            cmd = self.db_find_rows('Chassis', ('hostname', '=', hostname))
-            chassis = next(c for c in cmd.execute(check_error=True))
-        except StopIteration:
-            msg = _('Chassis with hostname %s does not exist') % hostname
-            raise RuntimeError(msg)
-        return (chassis.external_ids.get('datapath-type', ''),
-                chassis.external_ids.get('iface-types', ''),
-                self._get_chassis_physnets(chassis))
-
     def get_metadata_port_network(self, network):
         # TODO(twilson) This function should really just take a Row/RowView
         try:
