@@ -279,11 +279,11 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
     # The migration will run just once per neutron-server instance. If the lock
     # is held by some other neutron-server instance in the cloud, we'll attempt
     # to perform the migration every 10 seconds until completed.
-    # TODO(ihrachys): Remove the migration to stateless fips at some point.
+    # TODO(ihrachys): Remove the migration to stateful fips in Z+1.
     @periodics.periodic(spacing=10, run_immediately=True)
     @rerun_on_schema_updates
-    def migrate_to_stateless_fips(self):
-        """Perform the migration from stateful to stateless Floating IPs. """
+    def migrate_to_stateful_fips(self):
+        """Perform the migration from stateless to stateful Floating IPs. """
         # Only the worker holding a valid lock within OVSDB will perform the
         # migration.
         if not self.has_lock:
@@ -293,7 +293,7 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
         nb_sync = ovn_db_sync.OvnNbSynchronizer(
             self._ovn_client._plugin, self._nb_idl, self._ovn_client._sb_idl,
             None, None)
-        nb_sync.migrate_to_stateless_fips(admin_context)
+        nb_sync.migrate_to_stateful_fips(admin_context)
         raise periodics.NeverAgain()
 
     # The migration will run just once per neutron-server instance. If the lock
@@ -807,6 +807,39 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
                     txn.add(cmd)
         raise periodics.NeverAgain()
 
+    # TODO(ralonsoh): Remove this in the Z+4 cycle
+    @periodics.periodic(spacing=600, run_immediately=True)
+    def update_port_virtual_type(self):
+        """Set type=virtual to those ports with parents
+        Before LP#1973276, any virtual port with "device_owner" defined, lost
+        its type=virtual. This task restores the type for those ports updated
+        before the fix https://review.opendev.org/c/openstack/neutron/+/841711.
+        """
+        if not self.has_lock:
+            return
+
+        context = n_context.get_admin_context()
+        cmds = []
+        for lsp in self._nb_idl.lsp_list().execute(check_error=True):
+            if lsp.type != '':
+                continue
+
+            port = self._ovn_client._plugin.get_port(context, lsp.name)
+            for ip in port.get('fixed_ips', []):
+                if utils.get_virtual_port_parents(
+                        self._nb_idl, ip['ip_address'], port['network_id'],
+                        port['id']):
+                    cmds.append(self._nb_idl.db_set(
+                        'Logical_Switch_Port', lsp.uuid,
+                        ('type', ovn_const.LSP_TYPE_VIRTUAL)))
+                    break
+
+        if cmds:
+            with self._nb_idl.transaction(check_error=True) as txn:
+                for cmd in cmds:
+                    txn.add(cmd)
+        raise periodics.NeverAgain()
+
 
 class HashRingHealthCheckPeriodics(object):
 
