@@ -105,7 +105,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         self.sync_port_dns_records(ctx)
         self.sync_acls(ctx)
         self.sync_routers_and_rports(ctx)
-        self.migrate_to_stateless_fips(ctx)
+        self.migrate_to_stateful_fips(ctx)
         self.sync_port_qos_policies(ctx)
         self.sync_fip_qos_policies(ctx)
 
@@ -916,7 +916,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                 if self.mode == SYNC_MODE_REPAIR:
                     try:
                         # Create the missing port in both Neutron and OVN.
-                        LOG.warning('Creating missing metadadata port in '
+                        LOG.warning('Creating missing metadata port in '
                                     'Neutron and OVN for network %s',
                                     net['id'])
                         self._ovn_client.create_metadata_port(ctx, net)
@@ -1183,14 +1183,13 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                     txn.add(self.ovn_api.pg_add_ports(
                         utils.ovn_port_group_name(sg), port['id']))
 
-    def migrate_to_stateless_fips(self, ctx):
-        # This routine will set options:stateless=true for all dnat_and_snats
-        # that belong to neutron fips.
+    def migrate_to_stateful_fips(self, ctx):
+        # This routine will clear options:stateless=true for all dnat_and_snats
+        # that belong to neutron fips. Since we don't set any other options,
+        # just clear the whole column.
         with self.ovn_api.transaction(check_error=True) as txn:
-            for nat in self.ovn_api.get_all_stateful_fip_nats():
-                txn.add(self.ovn_api.db_set(
-                    'NAT', nat['_uuid'],
-                    ('options', {'stateless': 'true'})))
+            for nat in self.ovn_api.get_all_stateless_fip_nats():
+                txn.add(self.ovn_api.db_clear('NAT', nat['_uuid'], 'options'))
 
     def migrate_to_port_groups(self, ctx):
         # This routine is responsible for migrating the current Security
