@@ -29,6 +29,7 @@ from oslo_config import cfg
 from oslo_db import exception as os_db_exc
 from oslo_db.sqlalchemy import provision
 from oslo_log import log
+from oslo_utils import timeutils
 from oslo_utils import uuidutils
 
 from neutron.agent.linux import utils
@@ -421,9 +422,17 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
         self._start_ovn_northd()
 
     def add_fake_chassis(self, host, physical_nets=None, external_ids=None,
-                         name=None):
+                         name=None, enable_chassis_as_gw=False):
+        def append_cms_options(ext_ids, value):
+            if 'ovn-cms-options' not in ext_ids:
+                ext_ids['ovn-cms-options'] = value
+            else:
+                ext_ids['ovn-cms-options'] += ',' + value
+
         physical_nets = physical_nets or []
         external_ids = external_ids or {}
+        if enable_chassis_as_gw:
+            append_cms_options(external_ids, 'enable-chassis-as-gw')
 
         bridge_mapping = ",".join(["%s:br-provider%s" % (phys_net, i)
                                   for i, phys_net in enumerate(physical_nets)])
@@ -442,9 +451,11 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
             name, ['geneve'], '172.24.4.%d' % self._counter,
             external_ids=external_ids, hostname=host).execute(check_error=True)
         if self.sb_api.is_table_present('Chassis_Private'):
+            nb_cfg_timestamp = timeutils.utcnow_ts() * 1000
             self.sb_api.db_create(
                 'Chassis_Private', name=name, external_ids=external_ids,
-                chassis=chassis.uuid).execute(check_error=True)
+                chassis=chassis.uuid, nb_cfg_timestamp=nb_cfg_timestamp
+            ).execute(check_error=True)
         return name
 
     def del_fake_chassis(self, chassis, if_exists=True):
