@@ -140,34 +140,34 @@ class TestDBInconsistenciesPeriodics(testlib_api.SqlTestCaseLight,
                                                      migration_expected=False,
                                                      never_again=False)
 
-    def _test_migrate_to_stateless_fips_helper(
+    def _test_migrate_to_stateful_fips_helper(
             self, migration_expected, never_again):
         with mock.patch.object(ovn_db_sync.OvnNbSynchronizer,
-                               'migrate_to_stateless_fips') as mtsf:
+                               'migrate_to_stateful_fips') as mtsf:
             if never_again:
                 self.assertRaises(periodics.NeverAgain,
-                                  self.periodic.migrate_to_stateless_fips)
+                                  self.periodic.migrate_to_stateful_fips)
             else:
-                self.periodic.migrate_to_stateless_fips()
+                self.periodic.migrate_to_stateful_fips()
 
             if migration_expected:
                 mtsf.assert_called_once_with(mock.ANY)
             else:
                 mtsf.assert_not_called()
 
-    def test_migrate_to_stateless_fips(self):
+    def test_migrate_to_stateful_fips(self):
         # Check normal migration path: if the migration has to be done, it will
         # take place and won't be attempted in the future.
-        self._test_migrate_to_stateless_fips_helper(migration_expected=True,
-                                                    never_again=True)
+        self._test_migrate_to_stateful_fips_helper(migration_expected=True,
+                                                   never_again=True)
 
-    def test_migrate_to_stateless_fips_no_lock(self):
+    def test_migrate_to_stateful_fips_no_lock(self):
         with mock.patch.object(maintenance.DBInconsistenciesPeriodics,
                                'has_lock', mock.PropertyMock(
                                    return_value=False)):
             # Check that if this worker doesn't have the lock, it won't
             # perform the migration and it will try again later.
-            self._test_migrate_to_stateless_fips_helper(
+            self._test_migrate_to_stateful_fips_helper(
                 migration_expected=False, never_again=False)
 
     def _test_fix_create_update_network(self, ovn_rev, neutron_rev):
@@ -589,3 +589,22 @@ class TestDBInconsistenciesPeriodics(testlib_api.SqlTestCaseLight,
             mock.call('Logical_Router_Port', 'lrp-port1', ('options', opt))]
         self.fake_ovn_client._nb_idl.db_set.assert_has_calls(
             expected_calls)
+
+    @mock.patch.object(utils, 'get_virtual_port_parents',
+                       return_value=[mock.ANY])
+    def test_update_port_virtual_type(self, *args):
+        nb_idl = self.fake_ovn_client._nb_idl
+        lsp0 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'name': 'lsp0', 'type': ''})
+        lsp1 = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'name': 'lsp1', 'type': constants.LSP_TYPE_VIRTUAL})
+        port0 = {'fixed_ips': [{'ip_address': mock.ANY}],
+                 'network_id': mock.ANY, 'id': mock.ANY}
+        nb_idl.lsp_list.return_value.execute.return_value = (lsp0, lsp1)
+        self.fake_ovn_client._plugin.get_port.return_value = port0
+
+        self.assertRaises(
+            periodics.NeverAgain, self.periodic.update_port_virtual_type)
+        expected_calls = [mock.call('Logical_Switch_Port', lsp0.uuid,
+                                    ('type', constants.LSP_TYPE_VIRTUAL))]
+        nb_idl.db_set.assert_has_calls(expected_calls)
